//
//  ApplMonitorController.m
//  GlowWorm
//
//  Created by Curtis Jones on 2006.12.22.
//  Copyright 2006 __MyCompanyName__. All rights reserved.
//

#import "ApplMonitorController.h"

#import <Common.h>
#import <Logger.h>
#import <GlowWorm.h>
#import <GlowWormRule.h>
#import <GlowWormPlugin.h>

#import "AM_ApplTableDataSource.h"





@implementation ApplMonitorController





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark initalization functions





- (id)init
{
	self = [super init];
	
	if (self) {
		mApplSrc = [[AM_ApplTableDataSource alloc] init];
	}
	
	return self;
}

- (void)dealloc
{
	[mApplSrc release];
	
	[super dealloc];
}

- (void)awakeFromNib
{
	GWLog(@"%s\n", __PRETTY_FUNCTION__);
	
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	if ([mMainWin setFrameUsingName:@"GlowWorm Plugin (Appl Monitor)"])
		GWLog(@"%s.. success doing setFrameUsingName for mMainWin", __PRETTY_FUNCTION__);
	else
		GWLog(@"%s.. failed to setFrameUsingName for mMainWin", __PRETTY_FUNCTION__);
	
	// load our sub-menu
	{
		mMenuMainMenu  = [[[NSMenu alloc] init] autorelease];
		mMenuMainItem  = [[[NSMenuItem alloc] init] autorelease];
		mMenuPrefsItem = [[[NSMenuItem alloc] init] autorelease];
		mMenuRulesItem = [[[NSMenuItem alloc] init] autorelease];
		
		[mMenuMainItem setTitle:@"Application Monitor"];
		[mMenuMainItem setEnabled:TRUE];
		
		[mMenuPrefsItem setTarget:self];
		[mMenuPrefsItem setAction:@selector(actionPreferences:)];
		[mMenuPrefsItem setTitle:@"Preferences..."];
		[mMenuPrefsItem setEnabled:TRUE];
		
		[mMenuRulesItem setTarget:self];
		[mMenuRulesItem setAction:@selector(actionMonitor:)];
		[mMenuRulesItem setTitle:@"Monitor Window"];
		[mMenuRulesItem setEnabled:TRUE];
		
		[[GlowWormPlugin pluginsMenu] addItem:mMenuMainItem];
		[[GlowWormPlugin pluginsMenu] setSubmenu:mMenuMainMenu forItem:mMenuMainItem];
		
		[mMenuMainMenu addItem:mMenuPrefsItem];
		[mMenuMainMenu addItem:mMenuRulesItem];
	}
	
	[mMainTbl setDataSource:mApplSrc];
	[mMainTbl setDelegate:self];
	[mMainTbl reloadData];
	
	{
		AM_ApplCell * aCell = [[AM_ApplCell alloc] init];
		[[mMainTbl tableColumnWithIdentifier:@"Main"] setDataCell:aCell];
	}
	
	[NSThread detachNewThreadSelector:@selector(updateMonitorWindowLoop) toTarget:self withObject:nil];
	
	[self startup];
	
	[pool release];
}

- (void)startup
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	//
	// register to be notified if any preference values 
	// change, and set the default values for those 
	// preferences if there is no current value.
	//
	{
		[[NSNotificationCenter defaultCenter] addObserver:self
																						 selector:@selector(actionPluginDisableCallback:)
																								 name:@"actionPluginDisable"
																							 object:nil];
		
		[[NSNotificationCenter defaultCenter] addObserver:self
																						 selector:@selector(actionPluginEnableCallback:)
																								 name:@"actionPluginEnable"
																							 object:nil];
	}
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(mainWindowSelection:) name:@"NSTableViewSelectionDidChangeNotification" object:mMainTbl];
	
	[self startupLoadRules];
	
	[mMainWin makeKeyAndOrderFront:self];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	GWLog(@"GlowWormController::observeValueForKeyPath:ofObject:change:context().. keyPath=%@", keyPath);
}

- (void)startupLoadRules
{
	/* notification of new socket connections (for beeps) */
	{
		GlowWormRule * rule = [GlowWormRule ruleWithOnOff:GWR_ONOFF_ON
																									log:0
																								 term:GWR_TERM_TEMP
																								 list:GWR_LIST_CONN
																								 type:GWR_TYPE_NONE
																								 call:GWR_CALL_NONE
																							 action:GWR_ACTION_NONE
																						 throttle:GWR_THROTTLE_NONE
																								stats:GWR_STATS_NONE
																						 statfreq:0
																						 position:GWR_LOCA_END
																						 category:GWR_CAT_MAIN
																						direction:0
																								proto:GWR_PROTO_ANY
																						 socketid:0
																					 connection:nil
																					application:nil];
		
		[rule setRestrict:[NSNumber numberWithUnsignedLong:GWS_ACTION_CONNECT | GWS_ACTION_DETACH | GWS_ACTION_LISTEN]];
		[rule setMesgSize:[NSNumber numberWithUnsignedLong:GWR_MESGSIZE_FULL]];
		
		[GlowWormRule insertRule:rule
								withBlocking:FALSE
								 callbackObj:self
								 callbackSel:@selector(connActionCallback:)
								 callbackCtx:nil];
	}
	
	[GlowWormSock selectSock:0 callbackObj:self callbackSel:@selector(connSelectCallback:)];
	
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark action functions





- (void)mainWindowSelection:(NSNotification *)aNotification
{
	GWLog(@"%s.. selected index = %d", __PRETTY_FUNCTION__, [mMainTbl selectedRow]);
	
	int row = [mMainTbl selectedRow];
	
	if (row < 0 || row >= [mApplSrc count])
		[mMainInfoTxt setEnabled:FALSE];
	else
		[mMainInfoTxt setEnabled:TRUE];
	
}

- (IBAction)actionPreferences:(id)sender
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	[mPrefWin makeKeyAndOrderFront:sender];
}

- (IBAction)actionMonitor:(id)sender
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	[mMainWin makeKeyAndOrderFront:sender];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark observer callback functions





- (void)actionPluginDisableCallback:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	[mMainWin saveFrameUsingName:@"GlowWorm Plugin (Appl Monitor)"];
	
	[mApplSrc delAll];
}

- (void)actionPluginEnableCallback:(NSNotification *)aNotification
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	[self startupLoadRules];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark kext callback functions





- (void)connSelectCallback:(SockTrans *)aTrans
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	GlowWormSock * sock = (GlowWormSock *)[aTrans body];
//GlowWormMesg * mesg = [aTrans mesg];
	
	if (sock == nil)
		return;
	
	GWLog(@"%s.. [%@] '%@'", __PRETTY_FUNCTION__, [sock getProcessId], [sock getApplication]);
	
	[mApplSrc addSock:sock];
	
	[mMainTbl reloadData];
}

- (void)connActionCallback:(SockTrans *)aTrans
{
	GWLog(@"%s", __PRETTY_FUNCTION__);
	
	GlowWormSock * sock = (GlowWormSock *)[aTrans body];
	GlowWormMesg * mesg = [aTrans mesg];
	
	if (sock == nil)
		return;
	
	GWLog(@"%s.. [%@] '%@'", __PRETTY_FUNCTION__, [sock getProcessId], [sock getApplication]);
	
	if (GWS_ACTION_CONNECT == [mesg arg2])
		[mApplSrc addSock:sock];
	else if (GWS_ACTION_DETACH == [mesg arg2])
		[mApplSrc delSock:sock];
	
	[mMainTbl reloadData];
}





///////////////////////////////////////////////////////////////////////////////

#pragma mark -
#pragma mark kext miscellaneous functions





- (void)updateMonitorWindowLoop
{
	NSAutoreleasePool * apool = [[NSAutoreleasePool alloc] init];
	
	int curApplCount = 0;
	int curSockCount = 0;
	int remOldAppCnt = 0;
	
	[mMainInfoTxt setStringValue:@"0 application and 0 connections"];
		
	while (TRUE)
	{
		BOOL doReload = FALSE;
		NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
		
		if (++remOldAppCnt >= 5) {
			remOldAppCnt = 0;
			
			if (0 != [mApplSrc delOldAppls])
				doReload = TRUE;
		}
		
		int acount = [mApplSrc count];
		int scount = [mApplSrc sockCount];
		
		if (curApplCount != acount || curSockCount != scount) {
			[mMainInfoTxt setStringValue:[NSString stringWithFormat:@"%d application%s and %d connection%s", acount, (acount != 1 ? "s":""), scount, (scount != 1 ? "s":"")]];
			curApplCount = acount;
			curSockCount = scount;
			doReload = TRUE;
		}
		
		if (doReload)
			[mMainTbl reloadData];
		
		sleep(1);
		[pool release];
	}
	
	[apool release];
}

@end
